#!/bin/sh
#-------------------------------------------------------------------------
# install.sh
#
# Description:
#	Installation script for Encina software packages, getting them
#	from file, tape, or floppy.
#
# Usage:
#	install_tools [tape|file|floppy [file_or_device_name [list_of_pkgs]]]
#
# (C) Copyright 1990,1991,1992,1993,1994 Transarc Corporation
# All Rights Reserved
#
# This file may not be reproduced in any form or modified without the express 
# written consent of Transarc Corporation.
#--------------------------------------------------------------------------

#-------------------------------------------------------------------------
# computeRevisionString
#
# Description:
#	Take the revision and date values filled in automatically by
#	RCS/cml and organize it nicely.
#
# Arguments:
#	None.
#
# Side Effects:
#	As described.
#-------------------------------------------------------------------------

computeRevisionString() {

    #
    # These raw revision/date string is filled in by RCS/cml.
    #
    instRevDate='$Revision: 1.2 $ $Date: 1994/10/06 14:22:40 $'

    #
    # We need to pull out the 2nd (Revision), 5th (Date) and
    # 6th (Time) fields.
    #
    revDateWordNum=1
    for i in $instRevDate
    do
	if   [ $revDateWordNum = 2 ]; then
	    myRev=$i
	elif [ $revDateWordNum = 5 ]; then
	    myDate=$i
	elif [ $revDateWordNum = 6 ]; then
	    myTime=$i
	fi
	revDateWordNum=`expr "$revDateWordNum" + 1`
    done

    #
    # Put the nfo together the way we want it.
    #
    instVersion="$myRev of $myDate $myTime"
}


#-------------------------------------------------------------------------
# echon
#
# Description:
#	For those systems which do not support echo -n . . .
#
# Arguments:
#	EN .... Zero or one if the fast no-CR stuff is unsupported or
#		supported, respectively.
#
# Side Effects:
#	As described.
#-------------------------------------------------------------------------

echon() {
 if [ $EN = "1" ]
 then
   echo "$*\c"
 else
   echo -n "$*"
 fi
}


#-------------------------------------------------------------------------
# mtInit
#
# Description:
#	Tune the use of the 'mt' command based on the type of of
#	architecture on which we are running.  On most systems, we
#	use 'mt -t'; however, we use 'mt -f' on HPs and 'tctl' on
#	RS/6000s.
#
# Arguments:
#	None.
#
# Side Effects:
#	As described.
#-------------------------------------------------------------------------

mtInit() {

    if [ -x /bin/hp9000s700 ]; then 
	MT='mt -t'
        EN=1
    elif [ -x /bin/tctl ]; then
	MT='tctl -f'
        EN=0
    else
	MT='mt -f'
	EN=0
    fi
}


#--------------------------------------------------------------------------
# blankLines
#
# Abstract:
#	Print as many blank lines as specified in our argument.
#
# Arguments:
#	numLines ...... If this is set in the environment, then it
#			indicates the number of lines that are to be
#			printed.  Otherwise, one blank line will be
#			echoed.
#
# Side Effects:
#	As described.
#--------------------------------------------------------------------------

blankLines() {

    #
    # Pull in our "parameter" from the environment (choose a default of 1).
    # Set up to iterate.
    #
    junk=${numLines:=1}
    newlines="$newlineChar"
    loopCnt=1

    #
    # Put together a string with as many newlines as desired blanks.
    #
    while [ $loopCnt -lt $numLines ]
    do
	newlines="$newlines${newlineChar}"
	loopCnt=`expr "$loopCnt" + 1`
    done

    #
    # Generate the blank lines by echoing the constructed string.
    #
    $blankLineEcho $newlines
}


#--------------------------------------------------------------------------
# clearScreen
#
# Abstract:
#	Give the screen a spit shine.  Use the 'clear' utility if it's
#	supported (i.e., the 'clearFcn' variable is set in the overall
#	initialization), otherwise use blank lines.
#
# Arguments:
#	None.
#
# Side Effects:
#	As described.
#--------------------------------------------------------------------------

clearScreen() {

    if [ "X$clearFcn" != "X" ]; then
	$clearFcn
    else
	numLines=24; blankLines
    fi
}


#--------------------------------------------------------------------------
# installationInit
#
# Abstract:
#	Give the screen a spit shine.  Use the 'clear' utility if it's
#	supported (i.e., the 'clearFcn' variable is set in the overall
#	initialization), otherwise use blank lines.
#
# Arguments:
#	None.
#
# Side Effects:
#	As described.
#--------------------------------------------------------------------------

installationInit() {
    #
    # Don't assume we support echo with built-in no-carriage-return.
    #
    EN=0

    #
    # Decide on which built-in screen clearing function we have,
    # if any.
    #
    unset clearFcn
    if [ -x /usr/bin/clear ]; then
	clearFcn="/usr/bin/clear"
    elif
        [ -x /bin/clear ]; then
	clearFcn="/bin/clear"
    elif
        [ -x /usr/ucb/clear ]; then
	clearFcn="/usr/ucb/clear"
    fi

    #
    # Set up our device type and 'tape drive' defaults (namely, none),
    # then pull in the command line parameters, if any.  The first two
    # things there are expected to be the device type and the 'tape
    # drive'.  Any additional things on the command line are package
    # numbers (or 'A' or 'Q').  Also, zero the file we'll use to collect
    # up all our selections.
    #
    unset devType
    unset tapeDrive
    unset selectionsMade
    selectedPkgsFile="/tmp/tk_packages.$$"
    sortedSelectedPkgsFile="/tmp/tk_packages.sorted.$$"
    cp /dev/null $selectedPkgsFile

    if [ $# -gt 0  ]; then
	case $1 in
	    tape | file | floppy)
		devType=$1
		shift
		### echo "devType set to $devType"
		;;

	    *)
		echo "Unknown device type: '$1'"
		exit 1
	esac

	#
	# Hey, could there be a device name here too?
	#
	if [ $# -gt 0  ]; then
	    tapeDrive=$1
	    shift
	    ### echo "tapeDrive set to $tapeDrive"

	    #
	    # Let's go for broke; are there package numbers on the
	    # command line too?
	    #
	    allPkgsSelected=0
	    if [ $# -gt 0  ]; then
		selectionsMade=0
		while [ $# -gt 0  ]; do
		    ### echo "xxx Storing package selection '$1'"
		    if [ "$1" = "A" -o $1 = "a" ]; then
			allPkgsSelected=1
		    fi
		    echo $1 >> $selectedPkgsFile
		    shift
		    selectionsMade=`expr $selectionsMade + 1`
		done
		### echo "xxx Contents of package selection file:"
		### cat $selectedPkgsFile
	    fi
	fi
    fi

    #
    # Set up the proper magtape management invocation for this architecture.
    #
    mtInit

    #
    # Get our full revision string.
    #
    computeRevisionString

    #
    # Make sure we can print out newlines correctly.
    #
    newlineChar='\n'
    if [ -x /usr/5bin/echo ]; then
	blankLineEcho="/usr/5bin/echo"
    else
	blankLineEcho="echo"
    fi
}


#---------------------------------- main ----------------------------------

#
# Initialize our little script (pass it all the command line args so
# they'll be preserved within the initialization function).
#
installationInit $*

#
# Print a welcome message if neither of devType nor tapeDrive have already
# been set.
#
if [ "X$devType" = "X" ] || [ "X$tapeDrive" = "X" ]; then
    clearScreen
    numLines=4; blankLines

    echo "			   E N C I N A   (R)"
    echo "	                   ================="
    echo
    echo "		      Package Installation Program"
    echo "		   Version $instVersion"
    echo "		   ----------------------------------"
    echo
    echo "This is the TXSeries Encina e-fix package installation script."
    echo "It allows you to install one or more deliverable packages from a tape,"
    echo "file, or floppy."
    echo
    echon "Please hit <return> when you are ready to continue:"
    read nothing

    numLines=2; blankLines
    echo "You will be asked the name of the file, tape, or floppy device"
    echo "from which you wish to install."
    echo
    echo "If installing from a tape, you should specify a NO-REWIND tape"
    echo "device.  If you cannot, please type the interrupt character (usually"
    echo "ctrl-c), identify the appropriate device, and re-run this script."
    echo
    echon "Please hit <return> when you are ready to continue:"
    read nothing

    numLines=2; blankLines
    echo "NOTE:"
    echo
    echo "This installation program assumes that you are currently in"
    echo "the base Encina directory, into which you wish to install the"
    echo "package(s)."
    echo
    echo "If this is not the case, please type the interrupt character"
    echo "(usually ctrl-c) and 'cd' to the appropriate directory."
    echo
    echon "Please hit <return> when you are ready to continue:"
    read nothing

    #
    # Ask for the tape device.
    #
    numLines=2; blankLines
    echo "Please enter the type and name of the device from which you want to"
    echo "install the package(s).  There are three types recognized:"
    echo
    echo "    tape"
    echo "    file"
    echo "    floppy"
    echo
    echo "Example: If you want to install from tape device /dev/nrst0, you"
    echo "should type:"
    echo
    echo "    tape /dev/nrst0"
    echo
    echo
    echon "Type & Device? " ; read devType tapeDrive
    echo
else
    echo
    echo "Encina E-fix Package Installation Program [$instVersion]"
    echo
    echo "    Installing from '$tapeDrive'"
    if [ "X$selectionsMade" != "X" ]; then
	#
	# Print out the number of selected packages; recognize 'A'
	# specially.
	#
	if [ $allPkgsSelected = 1 ]; then
	    echon "    All available packages selected"
	else
	    echon "    $selectionsMade selected package(s): "
	    while read pkgChoice
	    do
		echon "$pkgChoice "
	    done < $selectedPkgsFile
	fi
	echo
    fi
fi
echo

#
# Figure out if it's really a tape or if it's a file
#
if [ $devType = "tape" ]
then
    	installSource=tape
else
	installSource=file
fi

# 
# Find out if the file/tape exists.  Cannot use -f here to test for existence, 
# because hp700 says that a device is not a plain file.  Also, cannot use -e
# to test for existence, because HP doesn't give proper results.  use -r instead,
# to test if we can read the file. 
#

if [ ! -r $tapeDrive ]
then
	echo "install.sh: Cannot read specified $installSource $tapeDrive.  Exiting."
	exit 1
fi

#
# Rewind the tape, if it's honest-to-goodness a tape.
#
if [ $installSource = "tape" ]
then
	echo
	echo
	echo "Rewinding the tape."

	$MT $tapeDrive rewind

	if [ $? -ne 0 ]
	then
		echo "install.sh: Problem rewinding tape.  Exiting."
		exit 1
	fi

	#
	# Skip past install.sh
	#

	echo "Skipping past my own image on the tape."
	echo
	$MT $tapeDrive fsf 1
fi

#
# Get the TOC from the tape.  Be verbose only if the user has not
# made his or her package selections from the command line.
#
if [ "X$selectionsMade" = "X" ]; then
    echo "Extracting the Table of Contents from the $installSource."
    echo
fi
tar xvf $tapeDrive TABLE_OF_CONTENTS 2>&1 /dev/null
if [ $? -ne 0 ]; then
    echo "install.sh: Problem reading from $installSource.  Exiting."
    exit 1
fi

if [ $installSource = "tape" ]; then
    #
    # Must insure that we are positioned after the first 2 tar entries
    # (ie: install.sh and TOC).  To do this, rewind, and then advance 2.
    # note: we cannot assume that because we have un-tar'ed the TOC, we
    # now point to the first package file.  On the rios, this is true;
    # on the sparc, this is not so.  On other architectures, don't know. 
    # To be sure, we rewind, and advance 2. 
    #
    $MT $tapeDrive rewind 
    $MT $tapeDrive fsf 2
fi

#
# Compute the number of packages available from the chosen medium.
#
packages=`cat TABLE_OF_CONTENTS | sed -n \
	-e 's/^.*totals: \([0-9][0-9]*\)/\1/p' | cut -f1 -d\ `

#
# If the desired selections have not already been made from the command
# line, get them now from the user.
#
if [ "X$selectionsMade" = "X" ]; then
    #
    # Print the table of contents for the user to peruse.
    #
    numLines=2; blankLines
    cat TABLE_OF_CONTENTS | more

    echo
    echo "The list above describes the Encina packages that are available from this"
    echo "$installSource.  You may specify the set of packages to install by entering"
    echo "the associated number or numbers (separated by whitespace) at the prompt"
    echo "below.  You will continue to be prompted for package numbers until you"
    echo "either enter 'Q' or type a carriage return at the prompt.  You may also"
    echo "choose to install all of the available packages by entering 'A' at the"
    echo "prompt."
    echo

    choices=-1

    while [ $choices != "Q" -a $choices != "q"  ]; do
	echon "Enter number(s) [Q]: "
	read choices
	### echo "xxx Value of choices: '$choices'"
	if [ "X$choices" != "X" ]; then
	    ### echo "xxx Going through items in '$choices'"
	    for choice in $choices
	    do
		### echo "xxx Processing choice '$choice'"
		if [ $choice = "A" -o $choice = "a" ]; then
		    #
		    # Select all packages, then drop out of the loop.
		    #
		    choice=1
		    while [ $choice -le $packages ]
		    do
			echo $choice >> $selectedPkgsFile
			choice=`expr "$choice" + 1`
		    done
		    echo " "
		    echo "Installing all listed packages."
		    choices="Q"
		elif [ $choice = "Q" -o $choice = "q" ]; then
		    #
		    # The caller wants us to quit right now.
		    #
		    choices="Q"
		    break # from the 'for' loop
		else
		    #
		    # This is an honest-to-goodness numerical value.
		    #
		    echo $choice | egrep '^[0-9][0-9]*$' > /dev/null
		    if [ $choice != "Q" -a $choice != "q" -a $? -eq 0 ]; then
			#
			# Start counting packages from 1 
			#
			if [ $choice -ge 1 -a $choice -le $packages ]; then
			    echo $choice >> $selectedPkgsFile
			else
			    echo "Invalid choice: '$choice'."
			fi
		    else
			if [ $choice != "Q" -a $choice != "q" ]; then
			    echo "Invalid choice."
			fi
		    fi
		fi
	    done # with current choice

	    #
	    # If our choices variable has not already been set to 'Q', then
	    # we want to keep looping.
	    #
	    if [ "$choices" != 'Q' -a "$choices" != 'q' ]; then
		choices=-1
	    fi
	else
	    #
	    # No input from the user; we're done with choices.
	    #
	    choices="Q"
	fi

	### echo "xxx About to iterate"
    done # while not done with choices

    ### echo "xxx Done with choices"
else
    #
    # Packages were selected on the command line.  If 'A' was seen,
    # expand the list.
    #
    if [ $allPkgsSelected = 1 ]; then
	/bin/rm $selectedPkgsFile
	choice=1
	while [ $choice -le $packages ]
	do
	    echo $choice >> $selectedPkgsFile
	    choice=`expr "$choice" + 1`
	done
    fi
fi
	
#
# Sort the package numbers (removing duplicates) for correctness,
# leaving the output in a new temporary file.  Delete the original
# temporary file.
#
#   This sort correctly handles leading zeros on index numbers.
#   the sed guarantees that numbers less than 10 will
#   be preceded with leading zeroes.
#
sed 's/^\([0-9]\)$/0\1/' $selectedPkgsFile | sort -nu -o $sortedSelectedPkgsFile 
/bin/rm -f $selectedPkgsFile

#
# Now read each package number from the sorted temporary file and get the
# corresponding package from tape.
#
tapeIndex=1
skipIndex=0

while read packageNumber
do
	#
	# Just so you know, multiple tar archives on a tape
	# are strange beasts.  When writing them, an EOF
	# marker is placed after each tar archive.  This means
	# that when it comes time to unpack them, you have to
	# be sure that you skip the EOF marker and advance to
	# the next tar archive.  This is why we have to go through
	# all of the following pushups.
	#

	if [ $installSource = "tape" ]
	then

		#
		# Determine the number of EOF markers to skip.
		#

		skipIndex=`expr "$packageNumber" - "$tapeIndex" `
		if [ $skipIndex -gt 0 ]
		then
			echo
			echon "Skipping forward $skipIndex to the package "
			echo "with menu number $packageNumber."
			$MT $tapeDrive fsf $skipIndex
			tapeIndex=`expr "$tapeIndex" + "$skipIndex"`
		fi

		#
		# Skip over the specified number of EOF markers
		# and extract the archive from the tape.
		#

		echo
		echo "Installing package for menu number $packageNumber."
		echo


                dd if=$tapeDrive bs=1024 | uncompress -c | tar -xvf -
		if [ $? -ne 0 ]
		then
			echo "install.sh: Problem reading from tape.  Exiting."
			exit 2
		fi

		#
		# no need to Skip over the EOF marker (if using dd).
		#

		#
		# Advance our current tape position count.
		#

		tapeIndex=`expr "$tapeIndex" + 1`

		#this is a NOP
		cat TABLE_OF_CONTENTS | compress -f | uncompress -c > /dev/null
		cat TABLE_OF_CONTENTS | compress -f | uncompress -c > /dev/null
		cat TABLE_OF_CONTENTS | compress -f | uncompress -c > /dev/null
	else
		# FILE ARCHIVE
		# Note: this section illustrates a major difference 
		# btwn the tape and file archives.  In the file archives,
		# the included files are named tk_pkg_tar*.Z, while
		# in the tape archives, they have meaningful names.
		# So, when we extract from a file, we go by the numbers.

		#
		# Just extract the corresponding archive and run it
		# through tar to unpack it
		#
		echo " "
		echo "Installing package for menu number $packageNumber."

		# use two steps to unarchive.

		tar xvf $tapeDrive tk_pkg_tar$packageNumber.Z > /dev/null
		if [ $? -ne 0 ]
		then
			echon "install.sh: Problem extracting archive.  "
			echo "Exiting."
			exit 3
		fi

                uncompress -c tk_pkg_tar$packageNumber.Z | tar -xvf -
		if [ $? -ne 0 ]
		then
			echon "install.sh: Problem unpacking archive.  "
			echo "Exiting."
			exit 3
		fi

		/bin/rm -f tk_pkg_tar$packageNumber.Z

		tapeIndex=`expr "$tapeIndex" + 1`
	fi

done < $sortedSelectedPkgsFile


if [ $installSource = "tape" ]
then

	#
	# Rewind the tape
	#

	echo "Rewinding the tape."
	$MT $tapeDrive rewind
fi


#
# Clean up and go home.
#
echo
echo "Cleaning up."
/bin/rm -f $sortedSelectedPkgsFile

echo " "
echo "********** Done with package installation(s) **********"
numLines=2; blankLines

exit 0
